@@ -1,33 +1,15 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-#    Copyright (c) 2010 Dominique Dumont.
-#
-#    This file is part of Config-Model.
-#
-#    Config-Model is free software; you can redistribute it and/or
-#    modify it under the terms of the GNU Lesser Public License as
-#    published by the Free Software Foundation; either version 2.1 of
-#    the License, or (at your option) any later version.
-#
-#    Config-Model is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#    Lesser Public License for more details.
-#
-#    You should have received a copy of the GNU Lesser Public License
-#    along with Config-Model; if not, write to the Free Software
-#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-
 package Config::Model::Backend::Any ;
 {
-  $Config::Model::Backend::Any::VERSION = '1.265';
+  $Config::Model::Backend::Any::VERSION = '2.010';
 }
 
 use Carp;
@@ -35,6 +17,7 @@ use strict;
 use warnings ;
 use Config::Model::Exception ;
 use Any::Moose ;
+use namespace::autoclean;
 
 use File::Path;
 use Log::Log4perl qw(get_logger :levels);
@@ -167,7 +150,6 @@ sub write_data_and_comments {
     return $res ;
 }
 
-no Any::Moose ;
 __PACKAGE__->meta->make_immutable ;
 
 1;
@@ -180,7 +162,7 @@ Config::Model::Backend::Any - Virtual class for other backends
 
 =head1 VERSION
 
-version 1.265
+version 2.010
 
 =head1 SYNOPSIS
 
