@@ -1,55 +1,661 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-#    Copyright (c) 2005-2011 Dominique Dumont.
-#
-#    This file is part of Config-Model.
-#
-#    Config-Model is free software; you can redistribute it and/or
-#    modify it under the terms of the GNU Lesser Public License as
-#    published by the Free Software Foundation; either version 2.1 of
-#    the License, or (at your option) any later version.
-#
-#    Config-Model is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#    Lesser Public License for more details.
-#
-#    You should have received a copy of the GNU Lesser Public License
-#    along with Config-Model; if not, write to the Free Software
-#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-
 package Config::Model::CheckList ;
 {
-  $Config::Model::CheckList::VERSION = '1.265';
+  $Config::Model::CheckList::VERSION = '2.010';
 }
+use Any::Moose ;
+
 use Config::Model::Exception ;
 use Config::Model::IdElementReference ;
 use Config::Model::Warper ;
-use warnings ;
 use Carp;
-use strict;
 use Log::Log4perl qw(get_logger :levels);
 use Storable qw/dclone/;
 
-use base qw/Config::Model::AnyThing/ ;
+extends qw/Config::Model::AnyThing/ ;
 
 
 my $logger = get_logger("Tree::Element::CheckList") ;
 
+my @introspect_params = qw/refer_to computed_refer_to/ ;
+
+my @accessible_params =  qw/default_list upstream_default_list choice ordered/ ;
+my @allowed_warp_params = (@accessible_params, qw/level experience/);
+
+has [qw/backup data preset layered/] 
+    => ( is => 'rw', isa => 'HashRef' , default => sub { {} ;} ) ;
+has [qw/refer_to computed_refer_to/] 
+    => ( is => 'rw', isa => 'Maybe[HashRef]'  ) ;
+has [qw/refer_to/]  => ( is => 'rw', isa => 'Str'  ) ;
+has [qw/ordered_data choice/]
+    => ( is => 'rw', isa => 'ArrayRef' , default => sub { [] ;} ) ;
+has [qw/ordered/] => (is => 'ro', isa => 'Bool' ) ;
+
+has [qw/warp help/]  => (is => 'rw', isa => 'Maybe[HashRef]') ;
+
+around BUILDARGS => sub {
+    my $orig = shift ;
+    my $class = shift ;
+    my %args =  @_ ;
+    my %h = map { ( $_ => $args{$_}) ;} grep {defined $args{$_}} @allowed_warp_params;
+    return $class->$orig( backup => dclone (\%h), @_ );
+} ;
+
+
+sub BUILD {
+    my $self = shift;
+
+    if (defined $self->refer_to or defined $self->computed_refer_to) {
+	$self->submit_to_refer_to() ;
+    }
+
+    $self->set_properties() ; # set will use backup data
+
+    if (defined $self->warp) {
+        my $warp_info = $self->warp ;
+        $self->{warper} = Config::Model::Warper->new (
+            warped_object => $self,
+            %$warp_info ,
+            allowed => \@allowed_warp_params
+        ) ;
+    }
+
+    $self->cl_init ;
+
+    $logger->info("Created check_list element ".$self->element_name);
+    return $self ;
+}
+
+sub cl_init {
+    my $self = shift ;
+
+    $self->warp if ($self->{warp});
+
+    if (defined $self->{ref_object} ) {
+	my $level = $self->parent
+	  -> get_element_property(element => $self->{element_name},
+				  property => 'level',
+				 ) ;
+	$self->{ref_object}->get_choice_from_refered_to if $level ne 'hidden';
+    }
+}
+
+sub name {
+    my $self = shift ;
+    my $name =  $self->{parent}->name . ' '.$self->{element_name} ;
+    return $name ;
+}
+
+sub value_type { return 'check_list' ;} 
+
+
+# warning : call to 'set' are not cumulative. Default value are always
+# restored. Lest keeping track of what was modified with 'set' is
+# too hard for the user.
+sub set_properties {
+    my $self = shift ;
+
+    # cleanup all parameters that are handled by warp
+    map(delete $self->{$_}, @allowed_warp_params ) ;
+
+    if ($logger->is_debug()) {
+      my %h = @_;
+      my $keys = join(',',keys %h) ;
+      $logger->debug("set_properties called on $self->{element_name} with $keys");
+    }
+
+    # merge data passed to the constructor with data passed to set
+    my %args = (%{$self->{backup}},@_ );
+
+    # these are handled by Node or Warper
+    map { delete $args{$_} } qw/level experience/ ;
+
+    $self->{ordered} = delete $args{ordered} || 0 ;
+
+    if (defined $args{choice}) {
+	my @choice = @{ delete $args{choice} } ;
+	$self->{default_choice} = \@choice ;
+	$self->setup_choice( @choice ) ;
+    }
+
+    if (defined $args{default}) {
+	warn $self->name,": default param is deprecated, use default_list\n";
+	$args{default_list} = delete $args{default} ;
+    }
+
+    if (defined $args{default_list}) {
+	$self->{default_list} = delete $args{default_list} ;
+    }
+
+    # store default data in a hash (more convenient)
+    $self->{default_data} = { map { $_ => 1 } @{$self->{default_list}} } ;
+
+    if (defined $args{upstream_default_list}) {
+	$self->{upstream_default_list} = delete $args{upstream_default_list} ;
+    }
+
+    # store upstream default data in a hash (more convenient)
+    $self->{upstream_default_data} = 
+      { map { $_ => 1 } @{$self->{upstream_default_list}} } ;
+
+    Config::Model::Exception::Model
+	-> throw (
+		  object => $self,
+		  error => "Unexpected parameters :".join(' ', keys %args )
+		 ) 
+	  if scalar keys %args ;
+}
+
+sub setup_choice {
+    my $self = shift ;
+    my @choice = ref $_[0] ? @{$_[0]} : @_ ;
+
+    $logger->debug("CheckList $self->{element_name}: setup_choice with @choice");
+    # store all enum values in a hash. This way, checking
+    # whether a value is present in the enum set is easier
+    delete $self->{choice_hash} if defined $self->{choice_hash} ;
+    map {$self->{choice_hash}{$_} =  1;} @choice ;
+
+    $self->{choice}  = \@choice ;
+
+    # cleanup current preset and data if it does not fit current choices
+    foreach my $field (qw/preset data layered/) {
+	next unless defined $self->{$field} ; # do not create if not present
+	foreach my $item (keys %{$self->{$field}}) {
+	    delete $self->{$field}{$item} unless defined $self->{choice_hash}{$item} ;
+	}
+    }
+}
+
+# Need to extract Config::Model::Reference (used by Value, and maybe AnyId).
+
+
+sub submit_to_refer_to {
+    my $self = shift ;
+
+    if (defined $self->refer_to) {
+	$self->{ref_object} = Config::Model::IdElementReference -> new ( 
+	    refer_to   => $self->refer_to ,
+	    config_elt => $self,
+	) ;
+    }
+    elsif (defined $self->computed_refer_to) {
+	$self->{ref_object} = Config::Model::IdElementReference -> new (
+	    computed_refer_to => $self->computed_refer_to ,
+	    config_elt => $self,
+	) ;
+        my $var = $self->{computed_refer_to}{variables} ;
+
+	# refer_to registration is done for all element that are used as
+	# variable for complex reference (ie '- $foo' , {foo => '- bar'} )
+        foreach my $path (values %$var) {
+            # is ref during test case
+            #print "path is '$path'\n";
+            next if $path =~ /\$/ ; # next if path also contain a variable
+            my $master = $self->grab($path);
+            next unless $master->can('register_dependency');
+            $master->register_dependency($self) ;
+        }
+    }
+    else {
+	croak "checklist submit_to_refer_to: undefined refer_to or computed_refer_to" ;
+    }
+}
+
+sub setup_reference_choice {
+    my $self = shift ;
+    $self->setup_choice(@_) ;
+}
+
+
+
+sub get_type {
+    my $self = shift;
+    return 'check_list' ;
+}
+
+
+sub get_cargo_type { goto &cargo_type } 
+
+sub cargo_type {
+    my $self = shift ;
+    return 'leaf' ;
+}
+
+
+sub check {
+    my $self = shift ;
+    my @list = ref $_[0] eq 'ARRAY' ? @{$_[0]} : @_ ;
+    my %args = ref $_[0] eq 'ARRAY' ? @_[1,$#_] : (check => 'yes') ;
+    my $check = $self->_check_check ($args{check}) ;
+    
+    if (defined $self->{ref_object}) {
+	$self->{ref_object}->get_choice_from_refered_to ;
+    }
+
+    map {$self->store($_ , 1, $check ) } @list ;
+}
+
+# internal
+sub store {
+    my ($self, $choice, $value, $check) = @_;
+
+    my $inst = $self->instance ;
+
+    if ($value != 0 and $value != 1) {
+        Config::Model::Exception::WrongValue 
+	    -> throw ( error => "store: check item value must be boolean, "
+		              . "not '$value'.",
+		       object => $self) ;
+	return ;
+    }
+
+    my $ok = $self->{choice_hash}{$choice} || 0 ;
+
+    if ($ok ) {
+	if ($inst->preset) {
+	    $self->{preset}{$choice} = $value ;
+	}
+	elsif ($inst->layered) {
+	    $self->{layered}{$choice} = $value ;
+	}
+	else {
+	    $self->{data}{$choice} = $value ;
+	}
+	if ($self->{ordered} and $value) {
+	    my $ord = $self->{ordered_data} ;
+	    push @$ord,$choice unless scalar grep {$choice eq $_} @$ord ;
+	}
+	$self->notify_change unless $self->instance->initial_load;
+    }
+    elsif ($check eq 'yes')  {
+	my $err_str = "Unknown check_list item '$choice'. Expected '"
+                    . join("', '",@{$self->{choice}}) . "'" ;
+	$err_str .= "\n\t". $self->{ref_object}->reference_info 
+	  if defined $self->{ref_object};
+        Config::Model::Exception::WrongValue 
+	    -> throw ( error =>  $err_str , object => $self) ;
+    }
+}
+
+
+sub uncheck {
+    my $self = shift ;
+    my @list = ref $_[0] eq 'ARRAY' ? @{$_[0]} : @_ ;
+    my %args = ref $_[0] eq 'ARRAY' ? @_[1,$#_] : (check => 'yes') ;
+    my $check = $self->_check_check ($args{check}) ;
+
+    if (defined $self->{ref_object}) {
+	$self->{ref_object}->get_choice_from_refered_to ;
+    }
+
+    map {$self->store($_ , 0 , $check) } @list ;
+}
+
+
+my %accept_mode = map { ( $_ => 1) } 
+                      qw/custom standard preset default layered upstream_default/;
+
+
+sub is_checked {
+    my $self = shift ;
+    my $choice = shift ;
+    my %args = @_ ;
+    my $type = $args{mode} || '';
+    my $check = $self->_check_check($args{check}) ;
+
+    my $ok = $self->{choice_hash}{$choice} || 0 ;
+
+    if ($ok ) {
+
+	if ($type and not defined $accept_mode{$type}) {
+	    croak "is_checked: expected ", join (' or ',keys %accept_mode),
+	      "parameter, not $type" ;
+	}
+
+	my $dat = $self->{data} ;
+	my $pre = $self->{preset} ;
+	my $def = $self->{default_data} ;
+	my $ud  = $self->{upstream_default_data} ;
+	my $std_v = (defined $pre->{$choice} ? $pre->{$choice} : $def->{$choice}) || 0 ;
+
+	my $result 
+	  = $type eq 'custom'           ? ( $dat->{$choice} && ! $std_v ? 1 : 0 )
+          : $type eq 'preset'           ? $pre->{$choice}
+          : $type eq 'layered'          ? $self->{layered}{$choice}
+          : $type eq 'upstream_default' ? $ud ->{$choice}
+          : $type eq 'default'          ? $def->{$choice}
+          : $type eq 'standard'         ? $std_v
+          : defined $dat->{$choice}     ? $dat->{$choice}
+          :                               $std_v ;
+
+	return $result ;
+    }
+    elsif ($check eq 'yes')  {
+	my $err_str = "Unknown check_list item '$choice'. Expected '"
+                    . join("', '",@{$self->{choice}}) . "'" ;
+	$err_str .= "\n\t". $self->{ref_object}->reference_info 
+	  if defined $self->{ref_object};
+        Config::Model::Exception::WrongValue 
+	    -> throw ( error =>  $err_str ,
+		       object => $self) ;
+    }
+}
+
+
+# get_choice is always called when using check_list, so having a
+# warp safety check here makes sense
+
+sub get_choice {
+    my $self = shift ;
+
+    if (defined $self->{ref_object}) {
+	$self->{ref_object}->get_choice_from_refered_to ;
+    }
+
+    if (not defined $self->{choice}) {
+        my $msg = "check_list element has no defined choice. " . 
+	  $self->warp_error;
+	Config::Model::Exception::UnavailableElement
+	    -> throw (
+		      info => $msg,
+		      object => $self->parent,
+		      element => $self->element_name,
+		     ) ;
+    }
+
+    return @{ $self->{choice} } ;
+}
+
+sub get_default_choice {
+    my $self = shift ;
+    return @{$self->{default_choice} || [] } ;
+}
+
+sub get_builtin_choice {
+    carp "get_builtin_choice is deprecated, use get_upstream_default_choice";
+    goto &get_upstream_default_choice ;
+}
+
+sub get_upstream_default_choice {
+    my $self = shift ;
+    return @{$self->{upstream_default_data} || [] } ;
+}
+
+
+sub get_help {
+    my $self = shift ;
+    my $help = $self->{help} ;
+
+    return $help unless @_ ;
+
+    my $on_value = shift ;
+    return $help->{$on_value} if defined $help and defined $on_value ;
+
+    return ;
+}
+
+
+sub clear {
+    my $self = shift ;
+    map { $self->store($_ , 0 ) } $self->get_choice ;
+}
+
+sub clear_values { goto &clear ; } 
+
+
+my %old_mode = ( built_in_list => 'upstream_default_list',
+	       );
+
+sub get_checked_list_as_hash {
+    my $self = shift ;
+    my %args = @_ > 1 ? @_ : (mode => $_[0]) ;
+    my $mode = $args{mode}|| '';
+
+    foreach my $k (keys %old_mode) {
+	next unless $mode eq $k;
+	$mode = $old_mode{$k} ;
+	carp $self->location," warning: deprecated mode parameter: $k, ",
+	    "expected $mode\n";
+    }
+
+    if ($mode and not defined $accept_mode{$mode}) {
+	croak "get_checked_list_as_hash: expected ", 
+	    join (' or ',keys %accept_mode),
+		" parameter, not $mode" ;
+    }
+
+    # fill empty hash result missing data
+    my %h = map { $_ => 0 } $self->get_choice ;
+
+    my $dat = $self->{data} ;
+    my $pre = $self->{preset} ;
+    my $def = $self->{default_data} ;
+    my $lay = $self->{layered} ;
+    my $ud  = $self->{upstream_default_data} ;
+
+    # copy hash and return it
+    my %std = (%h, %$ud, %$lay, %$def, %$pre ) ;
+
+    # custom test must compare the whole list at once, not just one item at a time.
+    my %result 
+      = $mode eq 'custom'   ? ( ( grep { $dat->{$_} xor $std{$_} } keys %h ) ? (%h, %$pre ,%$dat) : %h )
+      : $mode eq 'preset'   ? (%h, %$pre )
+      : $mode eq 'layered'  ? (%h, %$lay )
+      : $mode eq 'upstream_default' ? (%h, %$ud) 
+      : $mode eq 'default'          ? (%h, %$def )
+      : $mode eq 'standard' ? %std
+      :                       (%h, %$def, %$pre, %$dat );
+
+    return wantarray ? %result : \%result;
+}
+
+
+sub get_checked_list {
+    my $self = shift ;
+
+    my %h = $self->get_checked_list_as_hash(@_) ;
+    my @good_order = $self->{ordered} ? @{$self->{ordered_data}} : sort keys %h ;
+    my @res = grep { $h{$_} } @good_order ;
+    return wantarray ? @res : \@res ;
+}
+
+
+sub fetch {
+    my $self = shift ;
+    return join (',', $self->get_checked_list(@_));
+}
+
+sub fetch_custom {
+    my $self = shift ;
+    return join (',', $self->get_checked_list('custom'));
+}
+
+sub fetch_preset {
+    my $self = shift ;
+    return join (',', $self->get_checked_list('preset'));
+}
+
+sub fetch_layered {
+    my $self = shift ;
+    return join (',', $self->get_checked_list('layered'));
+}
+
+sub get {
+    my $self = shift ;
+    my $path = shift ;
+    if ($path) {
+	Config::Model::Exception::User
+	    -> throw (
+		      object => $self,
+		      message => "get() called with a value with non-empty path: '$path'"
+		     ) ;
+    }
+    return $self->fetch(@_) ;
+}
+
+
+sub set {
+    my $self = shift ;
+    my $path = shift ;
+    my $list = shift ;
+
+    if ($path) {
+	Config::Model::Exception::User
+	    -> throw (
+		      object => $self,
+		      message => "set() called with a value with non-empty path: '$path'"
+		     ) ;
+    }
+
+    return $self->set_checked_list(split /,/,$list) ;
+}
+
+sub load {
+    my ($self,$string) = @_ ;
+    my @set = split /,/,$string;
+    foreach (@set) { s/^"|"$//g; s/\\"/"/g ;}
+    $self->set_checked_list(@set) ;
+}
+
+
+sub store_set { goto &set_checked_list } 
+
+sub set_checked_list {
+    my $self = shift ;
+    $logger->debug("called with @_");
+    $self->clear ;
+    $self->{ordered_data} = [ @_ ] ; # copy list
+    $self->check (@_) ;
+}
+
+
+sub set_checked_list_as_hash {
+    my $self = shift ;
+    my %check = ref $_[0] ? %{$_[0]} : @_ ;
+
+    $self->clear ; 
+
+    if (defined $self->{ref_object}) {
+	$self->{ref_object}->get_choice_from_refered_to ;
+    }
+
+    while (my ($key, $value) = each %check) {
+	$self->store($key,$value) ;
+    }
+}
+
+
+sub load_data {
+    my $self = shift ;
+    my $data  = shift ;
+
+    if (ref ($data)  eq 'ARRAY') {
+	$self->set_checked_list(@$data) ;
+    }
+    else {
+	Config::Model::Exception::LoadData
+	    -> throw (
+		      object => $self,
+		      message => "load_data called with non array ref arg",
+		      wrong_data => $data ,
+		     ) ;
+    }
+}
+
+
+sub swap {
+    my ($self,$a,$b) = @_ ;
+
+    foreach my $param ($a,$b) {
+	unless ($self->is_checked($param)) {
+	    my $err_str = "swap: choice $param must be set";
+	    Config::Model::Exception::WrongValue 
+		-> throw ( error =>  $err_str ,
+			   object => $self) ;
+	}
+    }
+
+    # perform swap in ordered list
+    foreach (@{$self->{ordered_data}}) {
+	if ($_ eq $a) {
+	    $_ = $b ;
+	}
+	elsif ($_ eq $b) {
+	    $_ = $a ;
+	}
+    }
+}
+
+
+sub move_up {
+    my ($self,$c) = @_ ;
+
+    unless ($self->is_checked($c)) {
+	my $err_str = "swap: choice $c must be set";
+	Config::Model::Exception::WrongValue 
+	    -> throw ( error =>  $err_str ,
+		       object => $self) ;
+    }
+
+    # perform move in ordered list
+    my $list = $self->{ordered_data} ;
+    for (my $i = 1 ; $i < @$list; $i++) {
+	if ($list->[$i] eq $c ) {
+	    $list->[$i] = $list->[$i - 1 ];
+	    $list->[$i - 1] = $c ;
+	    last;
+	}
+    }
+}
+
+
+sub move_down {
+    my ($self,$c) = @_ ;
+
+    unless ($self->is_checked($c)) {
+	my $err_str = "swap: choice $c must be set";
+	Config::Model::Exception::WrongValue 
+	    -> throw ( error =>  $err_str ,
+		       object => $self) ;
+    }
+
+    # perform move in ordered list
+    my $list = $self->{ordered_data} ;
+    for (my $i = 0 ; $i + 1 < @$list; $i++) {
+	if ($list->[$i] eq $c ) {
+	    $list->[$i] = $list->[$i + 1 ];
+	    $list->[$i + 1] = $c ;
+	    last;
+	}
+    }
+}
+
+# dummy to match Value call
+sub warning_msg { '' } 
+
+1;
+
+__END__
+
+
+=pod
+
 =head1 NAME
 
 Config::Model::CheckList - Handle check list element
 
 =head1 VERSION
 
-version 1.265
+version 2.010
 
 =head1 SYNOPSIS
 
@@ -114,85 +720,10 @@ of the configuration tree. See L</"Choice reference"> for details.
 
 =back
 
-=cut
-
 =head1 CONSTRUCTOR
 
 CheckList object should not be created directly.
 
-=cut
-
-my @introspect_params = qw/refer_to computed_refer_to/ ;
-
-my @accessible_params =  qw/default_list upstream_default_list choice ordered/ ;
-my @allowed_warp_params = (@accessible_params, qw/level experience/);
-
-sub new {
-    my $type = shift;
-    my %args = @_ ;
-
-    my $self = { data => {}, preset => {} , layered => {} , ordered_data => [] } ;
-    bless $self,$type;
-
-    foreach my $p (qw/element_name instance config_model/) {
-	$self->{$p} = delete $args{$p} or
-	  croak "$type->new: Missing $p parameter for element ".
-	    $self->{element_name} || 'unknown' ;
-    }
-
-    $self->_set_parent(delete $args{parent}) ;
-
-    my $warp_info = delete $args{warp} ;
-
-    $self->{help} = delete $args{help} ;
-
-    if (defined $args{refer_to} or defined $args{computed_refer_to}) {
-	$self->{choice} ||= [] ; # create empty choice
-	$self->{refer_to} = delete $args{refer_to} ;
-	$self->{computed_refer_to} = delete $args{computed_refer_to} ;
-	$self->submit_to_refer_to() ;
-    }
-
-    $self->{backup}  = dclone (\%args) ;
-
-    $self->set_properties() ; # set will use backup data
-
-    if (defined $warp_info) {
-        $self->{warper} = Config::Model::Warper->new (
-            warped_object => $self,
-            %$warp_info ,
-            allowed => \@allowed_warp_params
-        ) ;
-    }
-
-    $self->cl_init ;
-
-    $logger->info("Created check_list element $self->{element_name}");
-    return $self ;
-}
-
-sub cl_init {
-    my $self = shift ;
-
-    $self->warp if ($self->{warp});
-
-    if (defined $self->{ref_object} ) {
-	my $level = $self->parent
-	  -> get_element_property(element => $self->{element_name},
-				  property => 'level',
-				 ) ;
-	$self->{ref_object}->get_choice_from_refered_to if $level ne 'hidden';
-    }
-}
-
-sub name {
-    my $self = shift ;
-    my $name =  $self->{parent}->name . ' '.$self->{element_name} ;
-    return $name ;
-}
-
-sub value_type { return 'check_list' ;} 
-
 =head1 CheckList model declaration
 
 A check list element must be declared with the following parameters:
@@ -293,99 +824,6 @@ The following methods returns the checklist parameter :
 
 =back
 
-=cut
-
-# accessor to get some fields through methods (See man perltootc)
-foreach my $datum (@accessible_params, @introspect_params ) {
-    no strict "refs";       # to register new methods in package
-    *$datum = sub {
-	my $self= shift;
-	return $self->{$datum};
-    } ;
-}
-
-
-
-# warning : call to 'set' are not cumulative. Default value are always
-# restored. Lest keeping track of what was modified with 'set' is
-# too hard for the user.
-sub set_properties {
-    my $self = shift ;
-
-    # cleanup all parameters that are handled by warp
-    map(delete $self->{$_}, @allowed_warp_params ) ;
-
-    if ($logger->is_debug()) {
-      my %h = @_;
-      my $keys = join(',',keys %h) ;
-      $logger->debug("set_properties called on $self->{element_name} with $keys");
-    }
-
-    # merge data passed to the constructor with data passed to set
-    my %args = (%{$self->{backup}},@_ );
-
-    # these are handled by Node or Warper
-    map { delete $args{$_} } qw/level experience/ ;
-
-    $self->{ordered} = delete $args{ordered} || 0 ;
-
-    if (defined $args{choice}) {
-	my @choice = @{ delete $args{choice} } ;
-	$self->{default_choice} = \@choice ;
-	$self->setup_choice( @choice ) ;
-    }
-
-    if (defined $args{default}) {
-	warn $self->name,": default param is deprecated, use default_list\n";
-	$args{default_list} = delete $args{default} ;
-    }
-
-    if (defined $args{default_list}) {
-	$self->{default_list} = delete $args{default_list} ;
-    }
-
-    # store default data in a hash (more convenient)
-    $self->{default_data} = { map { $_ => 1 } @{$self->{default_list}} } ;
-
-    if (defined $args{upstream_default_list}) {
-	$self->{upstream_default_list} = delete $args{upstream_default_list} ;
-    }
-
-    # store upstream default data in a hash (more convenient)
-    $self->{upstream_default_data} = 
-      { map { $_ => 1 } @{$self->{upstream_default_list}} } ;
-
-    Config::Model::Exception::Model
-	-> throw (
-		  object => $self,
-		  error => "Unexpected parameters :".join(' ', keys %args )
-		 ) 
-	  if scalar keys %args ;
-}
-
-sub setup_choice {
-    my $self = shift ;
-    my @choice = ref $_[0] ? @{$_[0]} : @_ ;
-
-    $logger->debug("CheckList $self->{element_name}: setup_choice with @choice");
-    # store all enum values in a hash. This way, checking
-    # whether a value is present in the enum set is easier
-    delete $self->{choice_hash} if defined $self->{choice_hash} ;
-    map {$self->{choice_hash}{$_} =  1;} @choice ;
-
-    $self->{choice}  = \@choice ;
-
-    # cleanup current preset and data if it does not fit current choices
-    foreach my $field (qw/preset data layered/) {
-	next unless defined $self->{$field} ; # do not create if not present
-	foreach my $item (keys %{$self->{$field}}) {
-	    delete $self->{$field}{$item} unless defined $self->{choice_hash}{$item} ;
-	}
-    }
-}
-
-# Need to extract Config::Model::Reference (used by Value, and maybe AnyId).
-
 =head1 Choice reference
 
 The choice items of a check_list can be given by another configuration
@@ -466,72 +904,16 @@ example is admittedly convoluted):
 
 =back
 
-=cut
-
-sub submit_to_refer_to {
-    my $self = shift ;
-
-    if (defined $self->{refer_to}) {
-	$self->{ref_object} = Config::Model::IdElementReference 
-	  -> new ( refer_to   => $self->{refer_to} ,
-		   config_elt => $self,
-		 ) ;
-    }
-    elsif (defined $self->{computed_refer_to}) {
-	$self->{ref_object} = Config::Model::IdElementReference 
-	  -> new ( computed_refer_to => $self->{computed_refer_to} ,
-		   config_elt => $self,
-		 ) ;
-        my $var = $self->{computed_refer_to}{variables} ;
-
-	# refer_to registration is done for all element that are used as
-	# variable for complex reference (ie '- $foo' , {foo => '- bar'} )
-        foreach my $path (values %$var) {
-            # is ref during test case
-            #print "path is '$path'\n";
-            next if $path =~ /\$/ ; # next if path also contain a variable
-            my $master = $self->grab($path);
-            next unless $master->can('register_dependency');
-            $master->register_dependency($self) ;
-        }
-    }
-    else {
-	croak "checklist submit_to_refer_to: undefined refer_to or computed_refer_to" ;
-    }
-}
-
-sub setup_reference_choice {
-    my $self = shift ;
-    $self->setup_choice(@_) ;
-}
-
-
 =head1 Methods
 
 =head2 get_type
 
 Returns C<check_list>.
 
-=cut
-
-sub get_type {
-    my $self = shift;
-    return 'check_list' ;
-}
-
 =head2 cargo_type()
 
 Returns 'leaf'.
 
-=cut
-
-sub get_cargo_type { goto &cargo_type } 
-
-sub cargo_type {
-    my $self = shift ;
-    return 'leaf' ;
-}
-
 =head2 check ( ... )
 
 Set choice. Parameter is either a list of choices to set or 
@@ -542,62 +924,6 @@ a list ref and some optional parameter. I.e:
 C<check> parameter decide on behavior in case of invalid
 choice value: either die (if yes) or discard bad value (if skip)
 
-=cut
-
-sub check {
-    my $self = shift ;
-    my @list = ref $_[0] eq 'ARRAY' ? @{$_[0]} : @_ ;
-    my %args = ref $_[0] eq 'ARRAY' ? @_[1,$#_] : (check => 'yes') ;
-    my $check = $self->_check_check ($args{check}) ;
-    
-    if (defined $self->{ref_object}) {
-	$self->{ref_object}->get_choice_from_refered_to ;
-    }
-
-    map {$self->store($_ , 1, $check ) } @list ;
-}
-
-# internal
-sub store {
-    my ($self, $choice, $value, $check) = @_;
-
-    my $inst = $self->instance ;
-
-    if ($value != 0 and $value != 1) {
-        Config::Model::Exception::WrongValue 
-	    -> throw ( error => "store: check item value must be boolean, "
-		              . "not '$value'.",
-		       object => $self) ;
-	return ;
-    }
-
-    my $ok = $self->{choice_hash}{$choice} || 0 ;
-
-    if ($ok ) {
-	if ($inst->preset) {
-	    $self->{preset}{$choice} = $value ;
-	}
-	elsif ($inst->layered) {
-	    $self->{layered}{$choice} = $value ;
-	}
-	else {
-	    $self->{data}{$choice} = $value ;
-	}
-	if ($self->{ordered} and $value) {
-	    my $ord = $self->{ordered_data} ;
-	    push @$ord,$choice unless scalar grep {$choice eq $_} @$ord ;
-	}
-    }
-    elsif ($check eq 'yes')  {
-	my $err_str = "Unknown check_list item '$choice'. Expected '"
-                    . join("', '",@{$self->{choice}}) . "'" ;
-	$err_str .= "\n\t". $self->{ref_object}->reference_info 
-	  if defined $self->{ref_object};
-        Config::Model::Exception::WrongValue 
-	    -> throw ( error =>  $err_str , object => $self) ;
-    }
-}
-
 =head2 uncheck (...)
 
 Unset choice. Parameter is either a list of choices to unset or 
@@ -608,21 +934,6 @@ a list ref and some optional parameter. I.e:
 C<check> parameter decide on behavior in case of invalid
 choice value: either die (if yes) or discard bad value (if skip)
 
-=cut
-
-sub uncheck {
-    my $self = shift ;
-    my @list = ref $_[0] eq 'ARRAY' ? @{$_[0]} : @_ ;
-    my %args = ref $_[0] eq 'ARRAY' ? @_[1,$#_] : (check => 'yes') ;
-    my $check = $self->_check_check ($args{check}) ;
-
-    if (defined $self->{ref_object}) {
-	$self->{ref_object}->get_choice_from_refered_to ;
-    }
-
-    map {$self->store($_ , 0 , $check) } @list ;
-}
-
 =head2 is_checked( choice, [ check => yes|skip ] , [ mode => ... ])
 
 Return 1 if the given C<choice> was set. Returns 0 otherwise.
@@ -632,135 +943,20 @@ choice value: either die (if yes) or discard bad value (if skip)
 
 C<mode> is either: custom standard preset default layered upstream_default
 
-=cut
-
-my %accept_mode = map { ( $_ => 1) } 
-                      qw/custom standard preset default layered upstream_default/;
-
-
-sub is_checked {
-    my $self = shift ;
-    my $choice = shift ;
-    my %args = @_ ;
-    my $type = $args{mode} || '';
-    my $check = $self->_check_check($args{check}) ;
-
-    my $ok = $self->{choice_hash}{$choice} || 0 ;
-
-    if ($ok ) {
-
-	if ($type and not defined $accept_mode{$type}) {
-	    croak "is_checked: expected ", join (' or ',keys %accept_mode),
-	      "parameter, not $type" ;
-	}
-
-	my $dat = $self->{data} ;
-	my $pre = $self->{preset} ;
-	my $def = $self->{default_data} ;
-	my $ud  = $self->{upstream_default_data} ;
-	my $std_v = (defined $pre->{$choice} ? $pre->{$choice} : $def->{$choice}) || 0 ;
-
-	my $result 
-	  = $type eq 'custom'           ? ( $dat->{$choice} && ! $std_v ? 1 : 0 )
-          : $type eq 'preset'           ? $pre->{$choice}
-          : $type eq 'layered'          ? $self->{layered}{$choice}
-          : $type eq 'upstream_default' ? $ud ->{$choice}
-          : $type eq 'default'          ? $def->{$choice}
-          : $type eq 'standard'         ? $std_v
-          : defined $dat->{$choice}     ? $dat->{$choice}
-          :                               $std_v ;
-
-	return $result ;
-    }
-    elsif ($check eq 'yes')  {
-	my $err_str = "Unknown check_list item '$choice'. Expected '"
-                    . join("', '",@{$self->{choice}}) . "'" ;
-	$err_str .= "\n\t". $self->{ref_object}->reference_info 
-	  if defined $self->{ref_object};
-        Config::Model::Exception::WrongValue 
-	    -> throw ( error =>  $err_str ,
-		       object => $self) ;
-    }
-}
-
 =head2 get_choice
 
 Returns an array of all items names that can be checked (i.e.
 that can have value 0 or 1).
 
-=cut
-
-# get_choice is always called when using check_list, so having a
-# warp safety check here makes sense
-
-sub get_choice {
-    my $self = shift ;
-
-    if (defined $self->{ref_object}) {
-	$self->{ref_object}->get_choice_from_refered_to ;
-    }
-
-    if (not defined $self->{choice}) {
-        my $msg = "check_list element has no defined choice. " . 
-	  $self->warp_error;
-	Config::Model::Exception::UnavailableElement
-	    -> throw (
-		      info => $msg,
-		      object => $self->parent,
-		      element => $self->element_name,
-		     ) ;
-    }
-
-    return @{ $self->{choice} } ;
-}
-
-sub get_default_choice {
-    my $self = shift ;
-    return @{$self->{default_choice} || [] } ;
-}
-
-sub get_builtin_choice {
-    carp "get_builtin_choice is deprecated, use get_upstream_default_choice";
-    goto &get_upstream_default_choice ;
-}
-
-sub get_upstream_default_choice {
-    my $self = shift ;
-    return @{$self->{upstream_default_data} || [] } ;
-}
-
 =head2 get_help (choice_value)
 
 Return the help string on this choice value
 
-=cut
-
-sub get_help {
-    my $self = shift ;
-    my $help = $self->{help} ;
-
-    return $help unless @_ ;
-
-    my $on_value = shift ;
-    return $help->{$on_value} if defined $help and defined $on_value ;
-
-    return ;
-}
-
 =head2 clear
 
 Reset the check list (all items are set to 0) (can also be called as
 C<clear_values>)
 
-=cut
-
-sub clear {
-    my $self = shift ;
-    map { $self->store($_ , 0 ) } $self->get_choice ;
-}
-
-sub clear_values { goto &clear ; } 
-
 =head2 get_checked_list_as_hash ( [ custom | preset | standard | default ] )
 
 Returns a hash (or a hash ref) of all items. The boolean value is the
@@ -805,114 +1001,19 @@ model)
 
 =back
 
-=cut
-
-my %old_mode = ( built_in_list => 'upstream_default_list',
-	       );
-
-sub get_checked_list_as_hash {
-    my $self = shift ;
-    my %args = @_ > 1 ? @_ : (mode => $_[0]) ;
-    my $mode = $args{mode}|| '';
-
-    foreach my $k (keys %old_mode) {
-	next unless $mode eq $k;
-	$mode = $old_mode{$k} ;
-	carp $self->location," warning: deprecated mode parameter: $k, ",
-	    "expected $mode\n";
-    }
-
-    if ($mode and not defined $accept_mode{$mode}) {
-	croak "get_checked_list_as_hash: expected ", 
-	    join (' or ',keys %accept_mode),
-		" parameter, not $mode" ;
-    }
-
-    # fill empty hash result missing data
-    my %h = map { $_ => 0 } $self->get_choice ;
-
-    my $dat = $self->{data} ;
-    my $pre = $self->{preset} ;
-    my $def = $self->{default_data} ;
-    my $lay = $self->{layered} ;
-    my $ud  = $self->{upstream_default_data} ;
-
-    # copy hash and return it
-    my %std = (%h, %$ud, %$lay, %$def, %$pre ) ;
-
-    # custom test must compare the whole list at once, not just one item at a time.
-    my %result 
-      = $mode eq 'custom'   ? ( ( grep { $dat->{$_} xor $std{$_} } keys %h ) ? (%h, %$pre ,%$dat) : %h )
-      : $mode eq 'preset'   ? (%h, %$pre )
-      : $mode eq 'layered'  ? (%h, %$lay )
-      : $mode eq 'upstream_default' ? (%h, %$ud) 
-      : $mode eq 'default'          ? (%h, %$def )
-      : $mode eq 'standard' ? %std
-      :                       (%h, %$def, %$pre, %$dat );
-
-    return wantarray ? %result : \%result;
-}
-
 =head2 get_checked_list ( < mode >  )
 
 Returns a list (or a list ref) of all checked items (i.e. all items
 set to 1). 
 
-=cut
-
-sub get_checked_list {
-    my $self = shift ;
-
-    my %h = $self->get_checked_list_as_hash(@_) ;
-    my @good_order = $self->{ordered} ? @{$self->{ordered_data}} : sort keys %h ;
-    my @res = grep { $h{$_} } @good_order ;
-    return wantarray ? @res : \@res ;
-}
-
 =head2 fetch ( < mode > )
 
 Returns a string listing the checked items (i.e. "A,B,C")
 
-=cut
-
-sub fetch {
-    my $self = shift ;
-    return join (',', $self->get_checked_list(@_));
-}
-
-sub fetch_custom {
-    my $self = shift ;
-    return join (',', $self->get_checked_list('custom'));
-}
-
-sub fetch_preset {
-    my $self = shift ;
-    return join (',', $self->get_checked_list('preset'));
-}
-
-sub fetch_layered {
-    my $self = shift ;
-    return join (',', $self->get_checked_list('layered'));
-}
 =head2 get( path  [, < mode> ] )
 
 Get a value from a directory like path.
 
-=cut
-
-sub get {
-    my $self = shift ;
-    my $path = shift ;
-    if ($path) {
-	Config::Model::Exception::User
-	    -> throw (
-		      object => $self,
-		      message => "get() called with a value with non-empty path: '$path'"
-		     ) ;
-    }
-    return $self->fetch(@_) ;
-}
-
 =head2 set( path , values )
 
 Set a checklist with a directory like path. Since a checklist is a leaf, the path
@@ -920,31 +1021,6 @@ should be empty. The values are a comma separated list of checked items.
 
 Example : C<< $leaf->set('','A,C,Z') ; >>
 
-=cut
-
-sub set {
-    my $self = shift ;
-    my $path = shift ;
-    my $list = shift ;
-
-    if ($path) {
-	Config::Model::Exception::User
-	    -> throw (
-		      object => $self,
-		      message => "set() called with a value with non-empty path: '$path'"
-		     ) ;
-    }
-
-    return $self->set_checked_list(split /,/,$list) ;
-}
-
-sub load {
-    my ($self,$string) = @_ ;
-    my @set = split /,/,$string;
-    foreach (@set) { s/^"|"$//g; s/\\"/"/g ;}
-    $self->set_checked_list(@set) ;
-}
-
 =head2 set_checked_list ( item1, item2, ..)
 
 Set all passed items to checked (1). All other available items
@@ -955,18 +1031,6 @@ Example:
   # set cl to A=0 B=1 C=0 D=1
   $cl->set_checked_list('B','D')
 
-=cut
-
-sub store_set { goto &set_checked_list } 
-
-sub set_checked_list {
-    my $self = shift ;
-    $logger->debug("called with @_");
-    $self->clear ;
-    $self->{ordered_data} = [ @_ ] ; # copy list
-    $self->check (@_) ;
-}
-
 =head2 set_checked_list_as_hash ( A => 1, B => 1 )
 
 Set check_list items. Missing items in the given list of parameters
@@ -976,47 +1040,11 @@ The example ( A => 1, B => 1 ) above will give :
 
  A = 1 , B = 1, C = 0 , D = 0
 
-=cut
-
-sub set_checked_list_as_hash {
-    my $self = shift ;
-    my %check = ref $_[0] ? %{$_[0]} : @_ ;
-
-    $self->clear ; 
-
-    if (defined $self->{ref_object}) {
-	$self->{ref_object}->get_choice_from_refered_to ;
-    }
-
-    while (my ($key, $value) = each %check) {
-	$self->store($key,$value) ;
-    }
-}
-
 =head2 load_data ( list_ref )
 
 Load check_list as an array ref. Data is simply forwarded to
 L<set_checked_list>.
 
-=cut
-
-sub load_data {
-    my $self = shift ;
-    my $data  = shift ;
-
-    if (ref ($data)  eq 'ARRAY') {
-	$self->set_checked_list(@$data) ;
-    }
-    else {
-	Config::Model::Exception::LoadData
-	    -> throw (
-		      object => $self,
-		      message => "load_data called with non array ref arg",
-		      wrong_data => $data ,
-		     ) ;
-    }
-}
-
 =head1 Ordered checklist methods
 
 All the methods below are valid only for ordered checklists.
@@ -1025,92 +1053,14 @@ All the methods below are valid only for ordered checklists.
 
 Swap the 2 given choice in the list. Both choice must be already set.
 
-=cut
-
-sub swap {
-    my ($self,$a,$b) = @_ ;
-
-    foreach my $param ($a,$b) {
-	unless ($self->is_checked($param)) {
-	    my $err_str = "swap: choice $param must be set";
-	    Config::Model::Exception::WrongValue 
-		-> throw ( error =>  $err_str ,
-			   object => $self) ;
-	}
-    }
-
-    # perform swap in ordered list
-    foreach (@{$self->{ordered_data}}) {
-	if ($_ eq $a) {
-	    $_ = $b ;
-	}
-	elsif ($_ eq $b) {
-	    $_ = $a ;
-	}
-    }
-}
-
 =head1 move_up ( choice )
 
 Move the choice up in the checklist. 
 
-=cut
-
-sub move_up {
-    my ($self,$c) = @_ ;
-
-    unless ($self->is_checked($c)) {
-	my $err_str = "swap: choice $c must be set";
-	Config::Model::Exception::WrongValue 
-	    -> throw ( error =>  $err_str ,
-		       object => $self) ;
-    }
-
-    # perform move in ordered list
-    my $list = $self->{ordered_data} ;
-    for (my $i = 1 ; $i < @$list; $i++) {
-	if ($list->[$i] eq $c ) {
-	    $list->[$i] = $list->[$i - 1 ];
-	    $list->[$i - 1] = $c ;
-	    last;
-	}
-    }
-}
-
 =head1 move_down ( choice )
 
 Move the choice down in the checklist. 
 
-=cut
-
-sub move_down {
-    my ($self,$c) = @_ ;
-
-    unless ($self->is_checked($c)) {
-	my $err_str = "swap: choice $c must be set";
-	Config::Model::Exception::WrongValue 
-	    -> throw ( error =>  $err_str ,
-		       object => $self) ;
-    }
-
-    # perform move in ordered list
-    my $list = $self->{ordered_data} ;
-    for (my $i = 0 ; $i + 1 < @$list; $i++) {
-	if ($list->[$i] eq $c ) {
-	    $list->[$i] = $list->[$i + 1 ];
-	    $list->[$i + 1] = $c ;
-	    last;
-	}
-    }
-}
-
-# dummy to match Value call
-sub warning_msg { '' } 
-
-1;
-
-__END__
-
 =head1 AUTHOR
 
 Dominique Dumont, (ddumont at cpan dot org)
