@@ -9,13 +9,14 @@
 #
 package Config::Model::Instance;
 {
-  $Config::Model::Instance::VERSION = '2.005';
+  $Config::Model::Instance::VERSION = '2.010';
 }
 #use Scalar::Util qw(weaken) ;
 
 use Any::Moose ;
 use namespace::autoclean;
 use Any::Moose '::Util::TypeConstraints';
+use Any::Moose 'X::StrictConstructor' ;
 
 use File::Path;
 use Log::Log4perl qw(get_logger :levels);
@@ -35,6 +36,7 @@ use warnings::register ;
 use Carp qw/carp croak confess cluck/;
 
 my $logger = get_logger("Instance") ;
+my $change_logger = get_logger("Anything::Change") ;
 
 has [qw/root_class_name/] => (is => 'ro', isa => 'Str', required => 1) ;
 
@@ -298,6 +300,7 @@ sub write_root_dir {
 # FIXME: record changes to implement undo/redo ?
 sub notify_change {
     my $self = shift ;
+    $change_logger->debug("called for  instance ",$self->name) if $change_logger->is_debug ;
     $self->{needs_save} = 1;
 }
 
@@ -394,7 +397,7 @@ Config::Model::Instance - Instance of configuration tree
 
 =head1 VERSION
 
-version 2.005
+version 2.010
 
 =head1 SYNOPSIS
 
