@@ -9,7 +9,7 @@
 #
 package Config::Model::Value ;
 {
-  $Config::Model::Value::VERSION = '2.005';
+  $Config::Model::Value::VERSION = '2.010';
 }
 
 use Any::Moose;
@@ -29,6 +29,7 @@ use Storable qw/dclone/;
 extends qw/Config::Model::AnyThing/ ;
 
 my $logger = get_logger("Tree::Element::Value") ;
+my $change_logger = get_logger("Anything::Change") ;
 
 our $nowarning = 0; # global variable to silence warnings. Only used for tests
 
@@ -40,12 +41,12 @@ has fixes
 has [qw/warp compute computed_refer_to backup migrate_from/] 
     => (is => 'rw', isa => 'Maybe[HashRef]') ;
 
+has compute_obj => (is => 'ro', isa =>'Maybe[Config::Model::ValueComputer]', 
+    builder => '_build_compute_obj', lazy => 1) ;
+
 has [qw/write_as/]
     => (is => 'rw', isa => 'Maybe[ArrayRef]') ;
 
-has [qw/allow_compute_override/] 
-    => (is => 'rw', isa => 'Bool', default => 0 ); 
-
 has [qw/refer_to _data replace_follow/] 
     => (is => 'rw', isa => 'Maybe[Str]' ); 
     
@@ -54,7 +55,7 @@ has value_type => (is => 'rw', isa => 'ValueType');
 my @common_int_params  = qw/min max mandatory /;
 has \@common_int_params => (is => 'ro', isa => 'Maybe[Int]') ;
 
-my @common_hash_params = qw/replace assert warn_if_match warn_unless_match warn_unless/ ;
+my @common_hash_params = qw/replace assert warn_if_match warn_unless_match warn_unless help/ ;
 has \@common_hash_params => (is => 'ro', isa => 'Maybe[HashRef]') ;
 
 my @common_list_params = qw/choice/;
@@ -63,12 +64,27 @@ has \@common_list_params => (is => 'ro', isa => 'Maybe[ArrayRef]') ;
 my @common_str_params  = qw/default upstream_default convert match grammar warn/;
 has \@common_str_params => (is => 'ro', isa => 'Maybe[Str]') ;
 
-
 my @warp_accessible_params = (@common_int_params,@common_str_params, @common_list_params, @common_hash_params);
 
 my @allowed_warp_params = (@warp_accessible_params, qw/level experience help/);
 my @backup_list = (@allowed_warp_params, qw/migrate_from/) ;
 
+has compute_is_upstream_default => (is => 'ro', isa => 'Bool', lazy => 1, builder => '_compute_is_upstream_default') ;
+
+sub _compute_is_upstream_default {
+    my $self = shift;
+    return 0 unless defined $self->compute ;
+    return $self->compute_obj->use_as_upstream_default ;
+}
+
+has compute_is_default => (is => 'ro', isa => 'Bool', lazy => 1, builder => '_compute_is_default') ;
+
+sub _compute_is_default {
+    my $self = shift;
+    return 0 unless defined $self->compute ;
+    return ! $self->compute_obj->use_as_upstream_default ;
+}
+
 # as some information like experience must be backed up even though they are not
 # attributes, we cannot move below code in BUILD. (experience is actually used by node)
 around BUILDARGS => sub {
@@ -93,8 +109,6 @@ sub BUILD {
         ) ;
     }
 
-    $self->set_compute ( ) if $self->compute;
-
     $self->_init ;
 
     return $self ;
@@ -105,20 +119,19 @@ sub notify_change {
     my %args = @_ ;
     my $check_done = $args{check_done} || 0 ;
 
-    # $logger->debug("called while needs_check is ",$self->needs_check,
-	# " for ",$self->name) 
-	# if $logger->is_debug ;
+    
+    $change_logger->debug("called while needs_check is ",$self->needs_check,
+	" for ",$self->name) 
+	if $change_logger->is_debug ;
+    $self->needs_check(1) unless $check_done;
+    $self->SUPER::notify_change(%args) ;
 
-    unless ($self->needs_check) {
-	$self->needs_check(1) unless $check_done;
-	$self->SUPER::notify_change(%args) ;
-	# notify all warped or computed objects that depends on me 
-	foreach my $s ($self->get_depend_slave) {
-	    # $logger->debug("calling notify_change on slave ",$s->name) 
-		# if $logger->is_debug ;
-	    $s -> notify_change ;
-	}  ;
-    }
+    # notify all warped or computed objects that depends on me 
+    foreach my $s ($self->get_depend_slave) {
+	$change_logger->debug("calling notify_change on slave ",$s->name) 
+	    if $change_logger->is_debug ;
+	$s -> notify_change ;
+    }  ;
 }
 
 # internal method
@@ -161,61 +174,33 @@ sub set_default {
 }
 
 
-sub set_compute {
-    my ($self) = @_ ;
-
-    my $c_ref = $self->compute;
-
-    $self->allow_compute_override( delete $c_ref->{allow_override} )
-      if defined $c_ref->{allow_override} ;
-
-    if (ref($c_ref) eq 'HASH') {
-	$self->compute($c_ref) ;
-    }
-    else {
-	Config::Model::Exception::Model
-	    -> throw (
-		      object => $self,
-		      error => "Compute value must be a hash ref not $c_ref"
-		     ) ;
-    }
-
-    foreach my $item (qw/formula/) {
-	next if defined $self->{compute}{$item} ;
-	Config::Model::Exception::Model
-	    -> throw (
-		      object => $self,
-		      error => "Missing compute $item"
-		     ) ;
-    }
-
-    delete $self->{_compute} ;
-}
-
 # set up relation between objects required by the compute constructor
 # parameters
-sub submit_to_compute {
+sub _build_compute_obj {
     my $self = shift ;
 
     $logger->debug("called") ;
     
-    my $c_info = $self->{compute} ;
-    $self->{_compute} = Config::Model::ValueComputer
-      -> new (
-	      formula      => $c_info->{formula} ,
-	      variables    => $c_info->{variables} ,
-	      replace      => $c_info->{replace},
-	      value_object => $self ,
-	      value_type   => $self->{value_type},
-	      use_eval     => $c_info->{use_eval},
-	      undef_is     => $c_info->{undef_is},
-	     );
+    my $c_info = $self->compute ;
+    return unless $c_info ;
+    
+    my @compute_data ;
+    foreach ( keys %$c_info ) {
+	push @compute_data, $_, $c_info->{$_} if defined $c_info->{$_} ;
+    }
+
+    my $ret = Config::Model::ValueComputer->new(
+        @compute_data,
+        value_object => $self,
+        value_type   => $self->{value_type},
+    );
 
     # resolve any recursive variables before registration
-    my $v = $self->{_compute}->compute_variables ;
+    my $v = $ret->compute_variables ;
 
     $self->register_in_other_value( $v ) ;
     $logger->debug("done") ;
+    return $ret ;
 }
 
 sub register_in_other_value {
@@ -241,11 +226,7 @@ sub perform_compute {
     my $self = shift ;
     $logger->debug("called");
 
-    $self->submit_to_compute unless defined $self->{_compute} ;
-
-    confess unless ref($self->{_compute}) eq 'Config::Model::ValueComputer' ;
-
-    my $result = $self->{_compute} -> compute ;
+    my $result = $self->compute_obj -> compute ;
 
     #print "compute: result $result\n" ;
     # check if the computed result fits with the constraints of the
@@ -271,7 +252,7 @@ sub perform_compute {
 # internal, used to generate error messages
 sub compute_info {
     my $self = shift;
-    $self->{_compute} -> compute_info ;
+    $self->compute_obj-> compute_info ;
 }
 
 
@@ -291,16 +272,16 @@ sub set_migrate_from {
 		     ) ;
     }
 
-    $self->{_migrate_from} = Config::Model::ValueComputer
-      -> new (
-	      formula      => $mig_ref->{formula} ,
-	      variables    => $mig_ref->{variables} ,
-	      replace      => $mig_ref->{replace},
-	      use_eval     => $mig_ref->{use_eval},
-	      undef_is     => $mig_ref->{undef_is},
-	      value_object => $self ,
-	      value_type   => $self->{value_type}
-	     );
+    my @migrate_data ;
+    foreach (qw/formula variables replace use_eval undef_is/ ) {
+	push @migrate_data, $_, $mig_ref->{$_} if defined $mig_ref->{$_} ;
+    }
+
+    $self->{_migrate_from} = Config::Model::ValueComputer->new(
+        @migrate_data,
+        value_object => $self,
+        value_type   => $self->{value_type}
+    );
 
     # resolve any recursive variables before registration
     my $v = $self->{_migrate_from}->compute_variables ;
@@ -732,7 +713,7 @@ sub get_cargo_type {
 sub can_store {
     my $self= shift;
 
-    return not defined $self->{compute} || $self->allow_compute_override ;
+    return not defined $self->compute || $self->compute_obj->allow_user_override ;
 }
 
 
@@ -1183,8 +1164,8 @@ sub pre_store {
       if ($self->{warp} and defined $self->{warp_info} 
           and @{$self->{warp_info}{computed_master}});
 
-    if (defined $self->{compute} 
-	and not $self->allow_compute_override) {
+    if (defined $self->compute_obj
+	and not $self->compute_obj->allow_user_override) {
 	my $msg = 'assignment to a computed value is forbidden unless '
 	  .'compute -> allow_override is set.';
 	Config::Model::Exception::Model
@@ -1295,9 +1276,10 @@ sub fetch_custom {
 sub fetch_standard {
     my $self = shift ;
     my $pre_fetch = $self->_pre_fetch ;
-    my $v = defined $pre_fetch       ? $pre_fetch 
-          : defined $self->{layered} ? $self->{layered}
-          :                            $self->{upstream_default} ;
+    my $v = defined $pre_fetch                 ? $pre_fetch 
+          : defined $self->{layered}           ? $self->{layered}
+          : $self->compute_is_upstream_default ? $self->perform_compute
+          :                                      $self->{upstream_default} ;
     return $self->map_write_as($v) ;
 }
 
@@ -1334,7 +1316,7 @@ sub _pre_fetch {
     eval {
 	$std_value 
 	  = defined $self->{preset}        ? $self->{preset}
-          : defined $self->{compute}       ? $self->perform_compute 
+          : $self->compute_is_default      ? $self->perform_compute 
           :                                  $self->{default} ;
     };
 
@@ -1368,6 +1350,7 @@ sub _fetch {
     # always call to perform submit_to_warp
     my $pref = $self->_pre_fetch($mode, $check) ;
     my $known_upstream = defined $self->{layered} ? $self->{layered}
+		       : $self->compute_is_upstream_default ? $self->perform_compute 
                        :                            $self->{upstream_default} ;
     my $std = defined $pref ? $pref : $known_upstream ;
     my $data = $self->{data} ;
@@ -1418,10 +1401,11 @@ sub _fetch {
 	 : $mode eq 'layered'                   ? $self->{layered}
 	 : $mode eq 'upstream_default'          ? $self->{upstream_default}
 	 : $mode eq 'user'                      ? defined $data ? $data : $std
-         : $mode =~ /backend|allow_undef/       ? defined $data ? $data : $pref 
+         : $mode eq 'allow_undef'               ? defined $data ? $data : $std 
+         : $mode eq 'backend'                   ? defined $data ? $data : $pref 
          :                                      die "unexpected mode $mode " ;
 
-    $logger->debug("done in '$mode' mode for ".$self->location." -> " . Data::Dumper->Dumper(['res'],[$res])) 
+    $logger->debug("done in '$mode' mode for ".$self->location." -> $res") 
         if $logger->is_debug ;
 
     return $res ;
@@ -1626,7 +1610,7 @@ Config::Model::Value - Strongly typed configuration value
 
 =head1 VERSION
 
-version 2.005
+version 2.010
 
 =head1 SYNOPSIS
 
