@@ -1,36 +1,20 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2012 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-#    Copyright (c) 2005-2011 Dominique Dumont.
-#
-#    This file is part of Config-Model.
-#
-#    Config-Model is free software; you can redistribute it and/or
-#    modify it under the terms of the GNU Lesser Public License as
-#    published by the Free Software Foundation; either version 2.1 of
-#    the License, or (at your option) any later version.
-#
-#    Config-Model is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#    Lesser Public License for more details.
-#
-#    You should have received a copy of the GNU Lesser Public License
-#    along with Config-Model; if not, write to the Free Software
-#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-
 package Config::Model::Warper ;
 {
-  $Config::Model::Warper::VERSION = '1.260';
+  $Config::Model::Warper::VERSION = '2.010';
 }
 
 use Any::Moose ;
+use namespace::autoclean;
+
 use Log::Log4perl qw(get_logger :levels);
 use Data::Dumper ;
 use Storable qw/dclone/ ;
@@ -40,24 +24,31 @@ use Carp;
 
 has 'follow' => ( is => 'ro', isa => 'HashRef[Str]' , required => 1 );
 has 'rules'  => ( is => 'ro', isa => 'ArrayRef' , required => 1 );
-has 'warped_object'  => ( is => 'ro', isa => 'Config::Model::AnyThing' , 
-        weak_ref => 1, required => 1 );
 
-has '_values' => (  traits    => ['Hash'],
-             is        => 'ro',
-             isa       => 'HashRef[Str | Undef ]',
-             default   => sub { {} },
-             handles => { 
-                    _set_value => 'set' ,
-                    _get_value => 'get' ,
-                    _value_keys => 'keys' ,
-                } ,
-  ) ;
+has 'warped_object'  => ( 
+    is => 'ro', 
+    isa => 'Config::Model::AnyThing' , 
+    handles => [ 'notify_change' ],
+    weak_ref => 1, 
+    required => 1 
+);
+
+has '_values' => (
+    traits  => ['Hash'],
+    is      => 'ro',
+    isa     => 'HashRef[Str | Undef ]',
+    default => sub { {} },
+    handles => {
+        _set_value  => 'set',
+        _get_value  => 'get',
+        _value_keys => 'keys',
+    },
+);
 
 has _computed_masters => ( is => 'rw',  isa => 'HashRef' , init_arg => undef ) ;
 
-has _warped_nodes => ( is => 'rw',  isa => 'HashRef' , init_arg => undef, default   => sub { {} }, ) ;
-has _registered_values => ( is => 'rw',  isa => 'HashRef' , init_arg => undef, default   => sub { {} }, ) ;
+has [qw/_warped_nodes _registered_values/] =>
+  ( is => 'rw', isa => 'HashRef', init_arg => undef, default => sub { {} }, );
 
 has allowed => ( is => 'rw',  isa => 'ArrayRef' ) ;
 has morph => ( is => 'ro', isa => 'Bool' ) ;
@@ -588,7 +579,8 @@ sub warp_error {
     return $str ;
 }
 
-no Any::Moose ;
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 =head1 NAME
@@ -597,7 +589,7 @@ Config::Model::Warper - Warp tree properties
 
 =head1 VERSION
 
-version 1.260
+version 2.010
 
 =head1 SYNOPSIS
 
